import pyqcm
import numpy as np
import gc
from model_2D import model2D
import timeit
import psutil

# pyqcm.set_global_parameter('verb_ED')
# pyqcm.set_global_parameter('verb_Hilbert')
# pyqcm.set_global_parameter('Ground_state_method', 'L')
# pyqcm.set_global_parameter('PRIMME_algorithm', 1)
# pyqcm.set_global_parameter('Davidson_states', 2)
# pyqcm.set_global_parameter('PRIMME_preconditionning', 1)

# pyqcm.set_global_parameter('CSR_sym_store')
F = 'E'
Lx = 6
Ly = 2
model = model2D(Lx, Ly, sym=True)
pyqcm.set_global_parameter('Hamiltonian_format', F)
model.set_target_sectors('R0:N{:d}:S0'.format(Lx*Ly))
model.set_parameters("""
    U = 4
    t = 1
    mu = 0.5*U
""")

process = psutil.Process()

# Iset = []
def go():
    for i in range(5):
        I = pyqcm.model_instance(model, i)
        # Iset.append(I)
        T = timeit.default_timer()
        print("ground state : ", I.ground_state())
        T = timeit.default_timer() - T
        print('RAM Used (MB):', process.memory_info().rss/(1024*1024))
        print('instance is tracked : ', gc.is_tracked(I))
        del I
        # print('instance is finalized : ', gc.is_finalized(I))
        # print('instance is tracked : ', gc.is_tracked(I))
        print('GC : ', gc.get_count())
        gc.collect()
        print('RAM Used (MB):', process.memory_info().rss/(1024*1024))
        print('format {:s} : time = {:1.2f}'.format(F, T))
    print('go is finished')

go()




import pyqcm
import numpy as np
from model_1D import model1D
import timeit

pyqcm.set_global_parameter('verb_ED')
pyqcm.set_global_parameter('verb_Hilbert')
pyqcm.set_global_parameter('Ground_state_method', 'L')
pyqcm.set_global_parameter('PRIMME_algorithm', 1)
pyqcm.set_global_parameter('parallel_sectors')
# pyqcm.set_global_parameter('Davidson_states', 2)
# pyqcm.set_global_parameter('PRIMME_preconditionning', 1)

# pyqcm.set_global_parameter('CSR_sym_store')
F = 'E'
L = 14
model = model1D(L,sym=True)
pyqcm.set_global_parameter('Hamiltonian_format', F)
model.set_target_sectors('R0:N{:d}:S0'.format(L))
model.set_parameters("""
    U = 4
    t = 1
    mu = 0.5*U
""")
I = pyqcm.model_instance(model)
T = timeit.default_timer()
print("ground state : ", I.ground_state())
T = timeit.default_timer() - T
print('format {:s} : time = {:1.2f}'.format(F, T))





import pyqcm
CM = pyqcm.cluster_model( 4, 0)
clus = pyqcm.cluster(CM, ((0, 0, 0), ( 1, 0, 0), ( 0, 1, 0), ( 1, 1, 0)))
model = pyqcm.lattice_model('WSM_2x2', clus, (( 2, 0, 0), ( 0, 2, 0), (0, 0, 1)))
model.interaction_operator('U')
model.hopping_operator('t', (1, 0, 0), 1, tau=1, sigma=1)
model.hopping_operator('t', (0, 1, 0), 1, tau=2, sigma=2)
model.hopping_operator('t', (0, 0, 1), 1, tau=2, sigma=3)
model.hopping_operator('t', (0, 0, 0), -0.765367, tau=0, sigma=1)
model.hopping_operator('m', (0, 1, 0), -0.5, tau=1, sigma=1)
model.hopping_operator('m', (0, 0, 1), -0.5, tau=1, sigma=1)
model.hopping_operator('m', (0, 0, 0), 2, tau=0, sigma=1)

model.set_target_sectors(['R0:N4'])
model.set_parameters("""
U = 4
mu= 0.5*U
t = 1
m = 1.5
""")




import pyqcm
pyqcm.set_global_parameter('nosym')
from model_1D import model1D
model = model1D(4, sym=False)

mixing = 1

if mixing == 4:
    model.set_target_sectors(['R0:N4:S0'])
    model.set_parameters("""
    t=1
    tsi=1
    mu = 1
    """)

elif mixing == 3:
    model.set_target_sectors(['R0'])
    model.set_parameters("""
    t=1
    tsi=1
    S = 1e-9
    Hx = 1e-9
    mu = 1
    """)

elif mixing == 2:
    model.set_target_sectors(['R0:N4'])
    model.set_parameters("""
    t=1
    H = -1
    Hx = 1e-8 
    mu = 1
    """)

elif mixing == 1:
    model.set_target_sectors(['R0:S0'])
    model.set_parameters("""
    t=1
    tsi=1
    S = 1e-9
    mu = 1
    """)

elif mixing == 0:
    model.set_target_sectors(['R0:N4:S0'])
    model.set_parameters("""
    t=1
    mu = 1
    """)

I = pyqcm.model_instance(model)
I.spectral_function(wmax = 6, nk = 32, path='line', file='test_mixing_{:d}.pdf'.format(mixing))




import pyqcm
import numpy as np

# A simple 1D 4 site cluster
CM = pyqcm.cluster_model(4)
clus = pyqcm.cluster(CM, ((0, 0, 0),(1, 0, 0),(2, 0, 0),(3, 0, 0)))
model = pyqcm.lattice_model('1D_4', clus, ((4,0,0),))

model.interaction_operator('U', amplitude=1) # on site interaction
model.interaction_operator('V', link=( 1, 0, 0), amplitude=1) # NN interaction

# Defining an explicit operator for the "Hartree part" of the extended interaction
elems = [(( 0, 0, 0), ( 0, 0, 0), 1/np.sqrt(2)), ([3,0,0], ( 0, 0, 0), 1/np.sqrt(2))] # the 1/sqrt(2) factor ensures normalization
model.explicit_operator('Vm', elems, type='one-body', tau=0) # Vm is an on-site operator ---> tau=0
model.hopping_operator('t', ( 1, 0, 0), -1) 
model.density_wave('Delta', 'N', ( 1, 0, 0), amplitude=1) # A charge density wave with period 2 in position space



import pyqcm
import numpy as np
from model_1D import model1D
import timeit

# pyqcm.set_global_parameter('verb_ED')
# pyqcm.set_global_parameter('verb_Hilbert')
# pyqcm.set_global_parameter('max_dim_full', 6)
pyqcm.set_global_parameter('Ground_state_init_last')
# pyqcm.set_global_parameter('Ground_state_method', 'P')

# pyqcm.set_global_parameter('CSR_sym_store')
F = 'E'
L = 12
model = model1D(L,sym=False)
pyqcm.set_global_parameter('Hamiltonian_format', F)
model.set_target_sectors('R0:N{:d}:S0'.format(L))
model.set_parameters("""
    U = 4
    t = 1
    mu = 0.5*U
""")
                     
# I = pyqcm.model_instance(model)
# print("ground state : ", I.ground_state())
# model.set_parameter('U', 4.1)                  
# I = pyqcm.model_instance(model)
# print("ground state : ", I.ground_state())
# exit()


for u in np.arange(4,4.1,0.01):
    model.set_parameter('U', u)
    I = pyqcm.model_instance(model)
    T = timeit.default_timer()
    print("ground state : ", I.ground_state())
    T = timeit.default_timer() - T
    print('format {:s} : time = {:1.2f}'.format(F, T))





from pyqcm import *
from pyqcm.cdmft import *

import model_1D_2_4b

np.set_printoptions(precision=4, linewidth=512, suppress=True)

sec = 'R0:N6:S0'
set_target_sectors([sec])

set_parameters("""
    U=4
    mu=1
    t=1
    tb1_1=0.5
    tb2_1=0.5
    eb1_1=1
    eb2_1=-1
""")

pyqcm.solver='dvmc'
pyqcm.new_model_instance()
obs_mu = observable('mu_1', 1e-4, 4)
obs_t = observable('t_1', 1e-4, 4)
A = cdmft(varia=['tb1_1', 'tb2_1', 'eb1_1', 'eb2_1'], accur=1e-6, accur_hybrid=1e-8, accur_dist=1e-10, observables=[obs_mu, obs_t])
print(A)
